from __future__ import print_function
import sys
from argparse import ArgumentParser

from . import (
    VERSION,
    metadata,
)


def define_args():
    ap = ArgumentParser()
    ap.add_argument('-v', metavar="API_VERSION", dest='api_version',
                    default=metadata.DEFAULT_API_VERSION, help="Metadata API "
                    "version (default='{}')".format(metadata.DEFAULT_API_VERSION))
    ap.add_argument('--timeout', default=metadata.DEFAULT_TIMEOUT, type=int,
                    help='HTTP connection timeout in seconds (default: '
                    '{})'.format(metadata.DEFAULT_TIMEOUT))
    ap.add_argument('--version', action='store_true',
                    help='show the awsmeta version')
    ap.add_argument('-n', '--by-name', nargs=1, metavar="short-name",
                    help='Look up a named metadata value')
    ap.add_argument('--list-names', action='store_true',
                    help='Print a list of known named metadata keys')
    ap.add_argument('path', nargs='?')

    return ap


def resolve_args(ap, args=sys.argv[1:]):
    args = ap.parse_args(args)
    return args


def main():
    ap = define_args()
    args = resolve_args(ap)

    if args.version:
        print("v%s" % VERSION)
        sys.exit(0)

    if args.list_names:
        for name in metadata.ShortNames().list():
            print(name)
        sys.exit(0)

    if args.by_name:
        try:
            sn = metadata.ShortNames(args.api_version, args.timeout)
            print(sn.get(args.by_name[0]))
            sys.exit(0)
        except metadata.MetadataError as e:
            print(e, file=sys.stderr)
            sys.exit(1)

    try:
        print(metadata.path(
            path=args.path,
            api_version=args.api_version,
            timeout=args.timeout).strip(),
        )
    except metadata.MetadataError as e:
        print(e, file=sys.stderr)
        sys.exit(1)

if __name__ == '__main__':
    main()



from urllib2 import (
    urlopen,
    HTTPError,
    URLError,
)

BASEURL = 'http://169.254.169.254/'

DEFAULT_TIMEOUT = 2
DEFAULT_API_VERSION = 'latest'


class MetadataError(Exception):
    pass


def path(path=None, api_version=DEFAULT_API_VERSION, timeout=DEFAULT_TIMEOUT):
    if not api_version:
        api_version = 'latest'
    md_path = api_version
    if path:
        md_path = md_path + "/" + path
    try:
        u = urlopen(BASEURL + md_path, timeout=timeout)
    except HTTPError as e:
        if e.code == 404:
            raise MetadataError("Path not found: /%s" % path)
        else:
            raise MetadataError(e)
    except URLError as e:
        raise MetadataError(e)
    if not path:
        return "\n".join(map(lambda p: p.strip() + "/", u.readlines()))
    return u.read()


class ShortNames(object):
    '''Provide commonly-used metadata values by name'''
    names = {
        'az': '/meta-data/placement/availability-zone',
        'instance-id': '/meta-data/instance-id',
    }

    def __init__(self, api_version=None, timeout=DEFAULT_TIMEOUT):
        self.api_version = api_version
        self.timeout = timeout

    def list(self):
        return self.names.keys()

    def get(self, name):
        if name not in self.names:
            raise MetadataError('The shortname "{}" is not defined'.format(name))

        return path(self.names[name], self.api_version, self.timeout)



from __future__ import print_function
from setuptools import setup, find_packages
from awsmeta import VERSION
import os
import sys

with open('README.md') as f:
    long_description = f.read()


def per_user_data_files():
    target_dir = os.path.join('etc', 'bash_completion.d')
    if os.geteuid() == 0:
        target_dir = os.path.join('/', target_dir)
    else:
        print('''
        Non-root installation. If you want bash completion, you'll need to
        install it manually. The file is located at <install_dir>/{}/awsmeta
        '''.format(target_dir), file=sys.stderr)
    return [
        (target_dir, ['extras/bash_completion/awsmeta']),
    ]

setup(
    name="awsmeta",
    version=VERSION,
    description="A tool for querying the AWS metadata service from the CLI",
    long_description=long_description,
    author="Matthew Wedgwood",
    author_email="mw@rmn.com",
    url="https://github.com/slank/awsmeta",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Internet",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Internet",
    ],
    license="MIT",
    entry_points={
        "console_scripts": [
            'awsmeta = awsmeta.cli:main',
        ]
    },
    data_files=[
        ('etc/bash_completion.d', ('extras/bash_completion/awsmeta',)),
    ],
)
